//#############################################################################################################################################################################################################
#ifndef _IDIBUS_4DC_H_
#define _IDIBUS_4DC_H_
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#include "IdiBusSlave.h"
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#define IDIBUS_CUSTDEF_PSU_CH_STATE_DISABLE						0x00
#define IDIBUS_CUSTDEF_PSU_CH_STATE_ENABLE						0x01

#define	IDIBUS_CUSTDEF_FUNC_CODE_PSU_CH_SET_STATE 				0x01
#define IDIBUS_CUSTDEF_FUNC_CODE_PSU_CH_GET_VOLTAGE				0x02
#define IDIBUS_CUSTDEF_FUNC_CODE_PSU_CH_GET_CURRENT				0x03
#define IDIBUS_CUSTDEF_FUNC_CODE_PSU_CH_GET_STATUS				0x04
#define IDIBUS_CUSTDEF_FUNC_CODE_PSU_CH_GET_ALLDATA				0x05
#define IDIBUS_CUSTDEF_FUNC_CODE_PSU_CH_SET_TARGET_VOLTAGE		0x06
#define IDIBUS_CUSTDEF_FUNC_CODE_PSU_CH_SET_CURRENT_LIMIT		0x07
#define IDIBUS_CUSTDEF_FUNC_CODE_PSU_CH_SET_ALL					0x08
#define IDIBUS_CUSTDEF_FUNC_CODE_GET_INP_VOLTAGE				0x09
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#define IDIBUS_CUSTDEF_ERCODE_INVALID_DATA						100U
#define IDIBUS_CUSTDEF_ERCODE_NO_ACCESS							101U
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#define IDIBUS_CUSTDEF_PSUCH_GET_CV_MES_LENGTH					4U

#define IDIBUS_CUSTDEF_PSUCH_STBUF_ST_Pos						00U
#define IDIBUS_CUSTDEF_PSUCH_STBUF_ST_EN_Pos					0U
#define IDIBUS_CUSTDEF_PSUCH_STBUF_ST_PG_Pos					1U
#define IDIBUS_CUSTDEF_PSUCH_STBUF_ST_PERIPH_ERR_Pos			2U
#define IDIBUS_CUSTDEF_PSUCH_STBUF_ST_OVERCURRENT_ERR_Pos		3U
#define IDIBUS_CUSTDEF_PSUCH_STBUF_ST_OVERVOLTAGE_ERR_Pos		4U
#define IDIBUS_CUSTDEF_PSUCH_STBUF_ST_OFFSTATELEAKAGE_ERR_Pos	5U
#define IDIBUS_CUSTDEF_PSUCH_STBUF_ST_MEMORY_ERR_Pos			6U
#define IDIBUS_CUSTDEF_PSUCH_STBUF_ST_INPV_ERR_Pos				7U
#define IDIBUS_CUSTDEF_PSUCH_STBUF_TARGET_VOLATGE_Pos			01U
#define IDIBUS_CUSTDEF_PSUCH_STBUF_CURRENT_LIMIT_Pos			03U
#define IDIBUS_CUSTDEF_PSUCH_STBUF_SIZE							05U

#define IDIBUS_CUSTDEF_PSUCH_STBUF_OUTPUT_VOLTAGE_Pos			IDIBUS_CUSTDEF_PSUCH_STBUF_SIZE
#define IDIBUS_CUSTDEF_PSUCH_STBUF_OUTPUT_CURRENT_Pos			(IDIBUS_CUSTDEF_PSUCH_STBUF_OUTPUT_VOLTAGE_Pos + 4)
#define IDIBUS_CUSTDEF_PSUCH_STALLBUF_SIZE						(IDIBUS_CUSTDEF_PSUCH_STBUF_OUTPUT_CURRENT_Pos + 4)
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#define IDIBUS_CUSTDEF_PSUINPV_STATE_Pos						00U
#define IDIBUS_CUSTDEF_PSUINPV_STATE_UV_Pos						0U
#define IDIBUS_CUSTDEF_PSUINPV_STATE_OV_Pos						1U
#define IDIBUS_CUSTDEF_PSUINPV_VOLTAGE_Pos						01U
#define IDIBUS_CUSTDEF_PSUINPV_MES_LENGTH						(IDIBUS_CUSTDEF_PSUINPV_VOLTAGE_Pos + 2)
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#define IDIBUS_PSU_CH_TARG_V_MIN_mV								1500UL
#define IDIBUS_PSU_CH_TARG_V_MAX_mV								48000UL
#define IDIBUS_PSU_CH_AVERAGE_CURRENT_LIMIT_MAX					10200UL
#define IDIBUS_PSU_CH_AVERAGE_CURRENT_LIMIT_MIN					100UL
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#define PSU_INV_UNDERVOLTAGE_ERR_THR							10000U
#define PSU_INV_OVERVOLTAGE_ERR_THR								50000U
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
enum IDIBUS_MODULE_DEVICES_LIST {
	IDIBUS_PSU_CNTRL_DEV = 0x00,
	IDIBUS_INPV_CNTRL_DEV,
	IDIBUS_DEVICES_NUMBER
};
enum {
	IDIBUS_4DC_CH1_NUM = 0,
	IDIBUS_4DC_CH2_NUM,
	IDIBUS_4DC_CH3_NUM,
	IDIBUS_4DC_CH4_NUM,
	IDIBUS_4DC_CH_COUNT
	};
//=============================================================================================================================================================================================================
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
//=============================================================================================================================================================================================================
class IdiBus_4DC_PSUChannel : public IdiBusChannel {
	public:
		IdiBus_4DC_PSUChannel(void);
		int32_t  Voltage;
		int32_t  Current;
		uint16_t TargetVoltage;
		uint16_t CurrentLimit;
		struct {
			uint8_t Enable;
			uint8_t PowerGood;
			union {
				uint8_t Sum;
				struct {
					uint8_t PeripheryError : 1;
					uint8_t OvercurrentError : 1;
					uint8_t OvervoltageError : 1;
					uint8_t OffStateCurrentLeakage : 1;
					uint8_t MemoryError : 1;
					uint8_t InpVoltageError : 1;
					uint8_t Reserved : 2;
				} BF;
			} ERRORS;
		} STATUS;
};
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
class IdiBus_4DC_PSUCDevice : public IdiBusDevice {
	public :
		IdiBus_4DC_PSUCDevice(void *ParentModule) : IdiBusDevice (ParentModule) {}
			
		uint8_t getChStatus(uint8_t ChNum);
		void    getAllChStatus(void);

		uint8_t getChOutputVoltage(uint8_t ChNum);
		void    getAllChOutputVoltage(void);
				
		uint8_t getChOutputCurrent(uint8_t ChNum);
		void    getAllChOutputCurrent(void);

		uint8_t getChAllData(uint8_t ChNum);
		void    getAllChAllData(void);
				
		uint8_t setChState(uint8_t ChNum, uint8_t State);
		void    setAllChState(uint8_t State);
		void    setAllChState(uint8_t Ch1State, uint8_t Ch2State, uint8_t Ch3State, uint8_t Ch4State);

		uint8_t setChVoltage(uint8_t ChNum, uint16_t TargetVoltage);
		void    setAllChVoltage(uint16_t TargetVoltage);
		void    setAllChVoltage(uint16_t TV_CH1, uint16_t TV_CH2, uint16_t TV_CH3, uint16_t TV_CH4);

		uint8_t setChCurrentLimit(uint8_t ChNum, uint16_t CurrentLimit);
		void    setAllChCurrentLimit(uint16_t CurrentLimit);
		void    setAllChCurrentLimit(uint16_t CL_CH1, uint16_t CL_CH2, uint16_t CL_CH3, uint16_t CL_CH4);

		uint8_t setChVoltageAndCurrentLimit(uint8_t ChNum, uint16_t TargetVoltage, uint16_t CurrentLimit);
		void    setAllChVoltageAndCurrentLimit(uint16_t TargetVoltage, uint16_t CurrentLimit);
		void    setAllChVoltageAndCurrentLimit(uint16_t VCH1, uint16_t CCH1, uint16_t VCH2, uint16_t CCH2, uint16_t VCH3, uint16_t CCH3, uint16_t VCH4, uint16_t CCH4);
		
		IdiBus_4DC_PSUChannel getChData(uint8_t ChNum);
	protected :
		IdiBus_4DC_PSUChannel  CH[IDIBUS_4DC_CH_COUNT];
		virtual IdiBusChannel *getChannel(uint8_t Num);
		void                   parsePsuChState(uint8_t *State, uint8_t ChNum);
		void                   parsePsuChAllState(uint8_t *State, uint8_t ChNum);
		void                   setAllPsuChErr(uint8_t ErrCode);
			
};
//=============================================================================================================================================================================================================
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
//=============================================================================================================================================================================================================
class IdiBus_4DC_INPVChannel : public IdiBusChannel  {
	public:
	IdiBus_4DC_INPVChannel(void);
	uint16_t InpVoltage;
	union {
		uint8_t Sum;
		struct {
			uint8_t Undervoltage : 1;
			uint8_t Overvoltage : 1;
			uint8_t Reserved : 6;
		} BF;
	} STATUS;
};
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
class IdiBus_4DC_INPVCDevice : public IdiBusDevice {
	public :
		IdiBus_4DC_INPVCDevice (void *ParentModule) : IdiBusDevice (ParentModule) {}
		uint8_t                getInputVoltage(void);
		IdiBus_4DC_INPVChannel getChData(void);
	protected :
		IdiBus_4DC_INPVChannel INPV;
		virtual IdiBusChannel *getChannel(uint8_t Num);
};
//=============================================================================================================================================================================================================
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
//=============================================================================================================================================================================================================
class IdiBus_4DC : public IdiBusSlave {
	public:
 		IdiBus_4DC (IdiBusSerialLine *SlaveLine, uint8_t SlaveAddress) : IdiBusSlave(SlaveLine, SlaveAddress), PSUC_DEV(this), INPV_DEV(this)  {}
		IdiBus_4DC() : IdiBusSlave(), PSUC_DEV(this), INPV_DEV(this) {}
			
		virtual IdiBusDevice *getDevice(uint8_t Num);
		
		IdiBus_4DC_PSUCDevice  PSUC_DEV;
		IdiBus_4DC_INPVCDevice INPV_DEV;
};
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#endif //#define _IDIBUS_4DC_H_
//#############################################################################################################################################################################################################
