//#############################################################################################################################################################################################################
#ifndef _IDIBUS_SERIAL_COMDEFS_H_
#define _IDIBUS_SERIAL_COMDEFS_H_
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#include "IdiBusSystem.h"
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#if defined (__AVR_ATmega2560__) || (__AVR_ATmega1280__) 
#define USARTN_UCSRNA_RXC_Pos					7U
#define USARTN_UCSRNA_TXC_Pos					6U
#define USARTN_UCSRNA_UDR_Pos					5U
#define USARTN_UCSRNA_FE_Pos					4U
#define USARTN_UCSRNA_DOR_Pos					3U
#define USARTN_UCSRNA_UPE_Pos					2U
#define USARTN_UCSRNA_U2X_Pos					1U
#define USARTN_UCSRNA_MPCM_Pos					0U

#define USARTN_UCSRNB_RXCIE_Pos					7U
#define USARTN_UCSRNB_TXCIE_Pos					6U		
#define USARTN_UCSRNB_UDRIE_Pos					5U
#define USARTN_UCSRNB_RXEN_Pos					4U
#define USARTN_UCSRNB_TXEN_Pos					3U
#define USARTN_UCSRNB_UCSZ2_Pos					2U
#define USARTN_UCSRNB_RXB8_Pos					1U
#define USARTN_UCSRNB_TXB8_Pos					0U

#define USARTN_UCSRNC_UMSEL1					7U
#define USARTN_UCSRNC_UMSEL0					6U	
#define USARTN_UCSRNC_UPM1						5U
#define USARTN_UCSRNC_UPM0						4U
#define USARTN_UCSRNC_USBS						3U
#define USARTN_UCSRNC_UCSZ1						2U
#define USARTN_UCSRNC_UCSZ0						1U
#define USARTN_UCSRNC_UCPOL						0U

#define USARTN_TIM_TCCRA_WGMN1_Pos				1U

#define USARTN_TIM_TCCRB_PR1					1U
#define USARTN_TIM_TCCRB_PR8					2U
#define USARTN_TIM_TCCRB_PR64					3U
#define USARTN_TIM_TCCRB_PR256					4U
#define USARTN_TIM_TCCRB_PR1024					5U

#define USRTN_TIM_TIMSK_TOIE_Pos				0U
#define USRTN_TIM_TIMSK_OCIEA_Pos				1U
#define USRTN_TIM_TIMSK_OCIEB_Pos				2U
#define USRTN_TIM_TIMSK_OCIEC_Pos				3U

#define USARTN_TIM_TIFR_TOV_Pos					0U
#define USARTN_TIM_TIFR_OCFA_Pos				1U
#define USARTN_TIM_TIFR_OCFB_Pos				2U
#define USARTN_TIM_TIFR_OCFC_Pos				3U

#endif //#if defined (__AVR_ATmega2560__)
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#define IDIBUS_BAUDRATE_DSW_CODE_19200B			0x00
#define IDIBUS_BAUDRATE_DSW_CODE_500K			0x01
#define IDIBUS_BAUDRATE_DSW_CODE_2400B			0x02
#define IDIBUS_BAUDRATE_DSW_CODE_9600B			0x03
#define IDIBUS_BAUDRATE_DSW_CODE_115200B		0x04
#define IDIBUS_BAUDRATE_DSW_CODE_250K			0x05
#define IDIBUS_BAUDRATE_DSW_CODE_1M				0x06
#define IDIBUS_BAUDRATE_DSW_CODE_10M			0x07
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#define IDIBUS_SERIAL_TIMER_PR_REG				USARTN_TIM_TCCRB_PR64

#if   IDIBUS_SERIAL_TIMER_PR_REG == USARTN_TIM_TCCRB_PR1
#define IDIBUS_SERIAL_TIMER_PR_VALUE			1U
#elif IDIBUS_SERIAL_TIMER_PR_REG == USARTN_TIM_TCCRB_PR8
#define IDIBUS_SERIAL_TIMER_PR_VALUE			8U
#elif IDIBUS_SERIAL_TIMER_PR_REG == USARTN_TIM_TCCRB_PR64
#define IDIBUS_SERIAL_TIMER_PR_VALUE			64U
#elif IDIBUS_SERIAL_TIMER_PR_REG == USARTN_TIM_TCCRB_PR256
#define IDIBUS_SERIAL_TIMER_PR_VALUE			256U
#elif IDIBUS_SERIAL_TIMER_PR_REG == USARTN_TIM_TCCRB_PR1024
#define IDIBUS_SERIAL_TIMER_PR_VALUE			1024U
#else
#error "Idibus Serial: Incorrect Timer Prescaler value !"
#endif			//#if IDIBUS_T_CPU==CPU_ATmega2560
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#define IDIBUS_SERIAL_TX_BUF_SIZE				IDIMMES_MAX_MES_SIZE
#define IDIBUS_SERIAL_RX_BUF_SIZE				IDISMES_MAX_MES_SIZE
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
enum IDIBUS_SERIAL_REQUEST_TYPE {
	_IDIBUS_SERIAL_TX_RX_,
	_IDIBUS_SERIAL_TX_,
	//Alarm
	};
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
typedef struct {
	volatile uint8_t  *UCSRNA;
	volatile uint8_t  *UCSRNB;
	volatile uint8_t  *UCSRNC;
	volatile uint16_t *UBRRN;
	volatile uint8_t  *UDRN;
	} IDIBUS_SERIAL_USART_HW_TYPE;
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
typedef struct {
	volatile uint8_t  *TCCRNA;
	volatile uint8_t  *TCCRNB;
	volatile uint8_t  *TIMSKN;
	volatile uint8_t  *TIFRN;
	volatile uint16_t *OCRNA;
	volatile uint16_t *OCRNB;
	volatile uint16_t *TCNTN;
	} IDIBUS_SERIAL_USARTN_16BIT_TIMER;
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
typedef union {
	uint8_t SumVar;
	struct {
		uint8_t TimeoutERR : 1;
		uint8_t TxOverflow : 1;
		uint8_t RxOverflow : 1;
		uint8_t LLComERR   : 1;
		uint8_t TxZerolength : 1;
		uint8_t Reserved   : 3;
		} BF;
	} IDIBUS_SERIAL_ERROR_STATUS_TYPE;
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
typedef struct {
	uint8_t			  PIN_MASK;
	volatile uint8_t *PORT;
	volatile uint8_t *DDR;
	volatile uint8_t *PIN;
	} IDIBUS_SERIAL_GPIO;
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
typedef struct {
	uint8_t			  PIN_NUM;
	volatile uint8_t *PORT;
	} IDIBUS_SERIAL_GPIO_INIT;
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
typedef	struct {
	IDIBUS_SERIAL_GPIO S0_Gpio;
	IDIBUS_SERIAL_GPIO S1_Gpio;
	IDIBUS_SERIAL_GPIO S2_Gpio;
	} IDIBUS_SERIAL_BAUDRATE_DIPSW_TYPE;
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#endif //_IDIBUS_SERIAL_COMDEFS_H_
//#############################################################################################################################################################################################################
