//#############################################################################################################################################################################################################
#include "IdiBusSlave.h"
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
IdiBusSlave::IdiBusSlave( IdiBusSerialLine *SlaveLine, uint8_t SlaveAddress )
{
 this->Line = SlaveLine;
 this->Address = SlaveAddress;
 this->CRC_AVR.CRC_ErrAvrBufCounter = 0;
 this->CRC_AVR.CRC_ErrAvrBufPos = 0;
 this->CRC_AVR.CRC_ErrAvrBufSum = 0;
 this->CRC_MultipleError = 0;
 this->ModuleError = IDIER_NOPE;
}
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
IdiBusSlave::IdiBusSlave(void)
{
 this->Line = NULL;
 this->Address = 1;
 this->CRC_AVR.CRC_ErrAvrBufCounter = 0;
 this->CRC_AVR.CRC_ErrAvrBufPos = 0;
 this->CRC_AVR.CRC_ErrAvrBufSum = 0;
 this->CRC_MultipleError = 0;
 this->ModuleError = IDIER_NOPE;
}
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
void IdiBusSlave::Init(IdiBusSerialLine *SlaveLine, uint8_t SlaveAddress)
{
 this->Line = SlaveLine;
 this->Address = SlaveAddress;
}
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
//=============================================================================================================================================================================================================
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
//=============================================================================================================================================================================================================
uint8_t IdiBusSlave::SendRequestMMES(IdiBusMMES *MMES)
{
 if ( this->Line == NULL ) { return IDIERMST_TX_MES; }
 uint8_t RequestError = this->Line->sendMMES(MMES);
 this->CrcAvrErrorCalc(RequestError);
 if ( RequestError != IDIER_NOPE ) { return RequestError; }
 else if ( MMES->IsModuleError(MMES->RxData[0]) )
	{
	 if ( MMES->RxDataLength != 1 ) { return IDIERMST_INVALID_RX_REQUEST_FORMAT; }
	 else                           { this->ModuleError = MMES->RxData[0]; return MMES->RxData[0]; }
	}
 this->ModuleError = IDIER_NOPE;
 return IDIER_NOPE;
}
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
uint8_t IdiBusSlave::SendRequestMMESG(IdiBusMMESG *MMESG)
{
 if ( this->Line == NULL ) { return IDIERMST_TX_MES; }
 uint8_t RequestError = this->Line->sendMMESG(MMESG);
 this->CrcAvrErrorCalc(RequestError);
 if ( RequestError != IDIER_NOPE ) { return RequestError; }
 else if ( MMESG->IsGroupAddr() == 0 )
	{
	 if ( MMESG->IsModuleError(MMESG->RxData[0]) )
		{
		 if ( MMESG->RxDataLength != 1 ) { return IDIERMST_INVALID_RX_REQUEST_FORMAT; }
		 else                            { this->ModuleError = MMESG->RxData[0]; return MMESG->RxData[0]; }
		}
	}
 this->ModuleError = IDIER_NOPE;
 return IDIER_NOPE;
}
//=============================================================================================================================================================================================================
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
//=============================================================================================================================================================================================================
uint8_t IdiBusSlave::SendModuleSimpleSetCommand(uint8_t Command)
{
 IdiBusMMESG MMESG(this->Address);
 MMESG.ComFunc = Command;
 uint8_t RequestError = this->SendRequestMMESG(&MMESG);
 if ( RequestError != IDIER_NOPE ) { return RequestError; }
 else if (MMESG.RxDataLength != 1) { return IDIERMST_INVALID_RX_REQUEST_FORMAT; }
 else                              { return MMESG.RxData[0]; }
}
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
uint8_t IdiBusSlave::SendModuleSimpleSetCommand(uint8_t Command, uint8_t *TxData, uint16_t TxDataLength)
{
 IdiBusMMESG MMESG(this->Address, TxData, TxDataLength);
 MMESG.ComFunc = Command;
 uint8_t RequestError = this->SendRequestMMESG(&MMESG);
 if ( RequestError != IDIER_NOPE ) { return RequestError; }
 else if (MMESG.RxDataLength != 1) { return IDIERMST_INVALID_RX_REQUEST_FORMAT; }
 else                              { return MMESG.RxData[0]; }
}
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
void IdiBusSlave::CrcAvrErrorCalc(uint8_t ErrorCode)
{
 uint8_t ErrorBufCode;
 if ( ErrorCode == IDIERMST_CRC ) { ErrorBufCode = 1; }
 else						      { ErrorBufCode = 0; }
	 
 this->CRC_AVR.CRC_ErrAvrBufSum = this->CRC_AVR.CRC_ErrAvrBufSum + ErrorBufCode - this->CRC_AVR.CRC_ErrAvrBuf[this->CRC_AVR.CRC_ErrAvrBufPos];
 this->CRC_AVR.CRC_ErrAvrBuf[this->CRC_AVR.CRC_ErrAvrBufPos] = ErrorBufCode;
 if ( this->CRC_AVR.CRC_ErrAvrBufPos == (IDIER_MULTIPLE_CRC_AVRBUF_SIZE - 1) ) { this->CRC_AVR.CRC_ErrAvrBufPos = 0;   }
 else																		   { this->CRC_AVR.CRC_ErrAvrBufPos++;     }
 
 if ( this->CRC_AVR.CRC_ErrAvrBufCounter < IDIER_MULTIPLE_CRC_AVRBUF_SIZE )    { this->CRC_AVR.CRC_ErrAvrBufCounter++; }
 else
	{
	 if ( this->CRC_AVR.CRC_ErrAvrBufSum >= IDIER_MULTIPLE_CRC_AVRBUF_THR ) { this->CRC_MultipleError = 1; }
	 else                                                                   { this->CRC_MultipleError = 0; }
	}
}
//=============================================================================================================================================================================================================
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
//=============================================================================================================================================================================================================
uint8_t IdiBusSlave::c_Init(void)          { return this->SendModuleSimpleSetCommand(IDIMMES_COM_C_Init); }
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
uint8_t IdiBusSlave::c_Init(uint8_t Group) { return (Group > IDIBUS_GROUP_LAST_NUMBER) ? IDIERMST_INVALID_TX_PARAM : this->SendModuleSimpleSetCommand(IDIMMES_COM_C_Init, &Group, 1); }
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
uint8_t IdiBusSlave::c_ShtDown(void)       { return this->SendModuleSimpleSetCommand(IDIMMES_COM_C_ShtDown); }
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
uint8_t IdiBusSlave::c_Freeze(void)        { return this->SendModuleSimpleSetCommand(IDIMMES_COM_C_Freeze); }
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
uint8_t IdiBusSlave::c_Resume(void)        { return this->SendModuleSimpleSetCommand(IDIMMES_COM_C_Resume); }
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
uint8_t IdiBusSlave::c_DummyModule(void)   { return this->SendModuleSimpleSetCommand(IDIMMES_COM_C_DummyModule); }
//=============================================================================================================================================================================================================
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
//=============================================================================================================================================================================================================
uint8_t IdiBusSlave::c_SendTimeDate(IdiTimeFormat Time, IdiDateFormat Date)
{
 if ( (Time.isValid() == 0) || (Date.isValid() == 0) ) { return IDIERMST_INVALID_TX_PARAM; }
 uint8_t  TxData[IDIMMES_C_DATETIME_LENGTH];
 Time.getBufForMMES(&TxData[IDIMMES_C_DATETIME_TIME_Pos]);
 Date.getBufForMMES(&TxData[IDIMMES_C_DATETIME_DATE_Pos]);
 return this->SendModuleSimpleSetCommand(IDIMMES_COM_C_SendTimeDate, TxData, IDIMMES_C_DATETIME_LENGTH );
}
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
uint8_t IdiBusSlave::c_ReadDevFullSN_MS(void)
{
 IdiBusMMESG MMESG (this->Address);
 MMESG.ComFunc = IDIMMES_COM_C_ReadDevFullSN_MS;
 uint8_t RequestError = this->SendRequestMMESG(&MMESG);
 if      ( RequestError != IDIER_NOPE )                                                   { return RequestError; }
 else if ( MMESG.RxDataLength != (IDISN_FULL_LENGTH - IDISN_VARP_AES256_Length + 2 + 1) ) { return IDIERMST_INVALID_RX_REQUEST_FORMAT; }
 else if ( MMESG.RxData[0] != IDIER_NOPE )                                                { return MMESG.RxData[0]; }
 else
	{
	 this->ModuleST.parseSmesDataNoAes(&MMESG.RxData[1]);
	 return IDIER_NOPE;
	}
}
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
uint8_t IdiBusSlave::c_WriteSnIPv4IPv6(uint32_t IPv4, uint32_t IPv6_B3, uint32_t IPv6_B2, uint32_t IPv6_B1, uint32_t IPv6_B0)
{
 uint8_t TxData[IDISN_VARP_IPv4_Length + IDISN_VARP_IPv6_Length];
 TxData[0] = (uint8_t)(IPv4 >> 24);
 TxData[1] = (uint8_t)(IPv4 >> 16);
 TxData[2] = (uint8_t)(IPv4 >> 8);
 TxData[3] = (uint8_t)(IPv4);
 TxData[4] = (uint8_t)(IPv6_B3 >> 24);
 TxData[5] = (uint8_t)(IPv6_B3 >> 16);
 TxData[6] = (uint8_t)(IPv6_B3 >> 8);
 TxData[7] = (uint8_t)(IPv6_B3);
 TxData[8]  = (uint8_t)(IPv6_B2 >> 24);
 TxData[9]  = (uint8_t)(IPv6_B2 >> 16);
 TxData[10] = (uint8_t)(IPv6_B2 >> 8);
 TxData[11] = (uint8_t)(IPv6_B2);
 TxData[12] = (uint8_t)(IPv6_B1 >> 24);
 TxData[13] = (uint8_t)(IPv6_B1 >> 16);
 TxData[14] = (uint8_t)(IPv6_B1 >> 8);
 TxData[15] = (uint8_t)(IPv6_B1);
 TxData[16] = (uint8_t)(IPv6_B0 >> 24);
 TxData[17] = (uint8_t)(IPv6_B0 >> 16);
 TxData[18] = (uint8_t)(IPv6_B0 >> 8);
 TxData[19] = (uint8_t)(IPv6_B0);
 uint8_t RequestError = this->SendModuleSimpleSetCommand(IDIMMES_COM_C_WriteSnIPv4IPv6, TxData, IDISN_VARP_IPv4_Length + IDISN_VARP_IPv6_Length );
 if ( RequestError != IDIER_NOPE ) { return RequestError; }
 else 
	{
	 memcpy(this->ModuleST.SN.VAR.IPv4, &TxData[0], IDISN_VARP_IPv4_Length);
	 memcpy(this->ModuleST.SN.VAR.IPv6, &TxData[4], IDISN_VARP_IPv6_Length);
	 return IDIER_NOPE;
	}
}
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
uint8_t IdiBusSlave::c_WriteSnVerifyDates(IdiDateFormat VDate, IdiDateFormat VEXPDate)
{
 if ( (VDate.isValid() == 0) || (VEXPDate.isValid() == 0) ) { return IDIERMST_INVALID_TX_PARAM; }
 uint8_t  TxData[IDIDATE_FORMAT_LENGTH  * 2];
 VDate.getBufForMMES(&TxData[0]);
 VEXPDate.getBufForMMES(&TxData[IDIDATE_FORMAT_LENGTH]); 
 uint8_t RequestError = this->SendModuleSimpleSetCommand(IDIMMES_COM_C_WriteSnVerifyDates, TxData, IDIDATE_FORMAT_LENGTH  * 2 );
 if ( RequestError != IDIER_NOPE ) { return RequestError; }
 else
	{
	 this->ModuleST.SN.VAR.VerifDate    = VDate;
	 this->ModuleST.SN.VAR.VerifExpDate = VEXPDate;
	 return IDIER_NOPE;
	}
}
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
uint8_t IdiBusSlave::c_WriteSnAES256(uint8_t *AES256_key) { return this->SendModuleSimpleSetCommand(IDIMMES_COM_C_WriteSnAES256, AES256_key, IDISN_VARP_AES256_Length); }
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
uint8_t IdiBusSlave::c_CheckModuleLongOp(void)
{
 IdiBusMMESG MMESG (this->Address);
 MMESG.ComFunc = IDIMMES_COM_C_CheckModuleLongOp;
 uint8_t RequestError = this->SendRequestMMESG(&MMESG);
 if      ( RequestError != IDIER_NOPE )                            { return RequestError; }
 else if ( MMESG.RxDataLength != (IDILONGOP_MES_DATA_LENGTH + 1) ) { return IDIERMST_INVALID_RX_REQUEST_FORMAT; }
 else if ( MMESG.RxData[0] != IDIER_NOPE )                         { return MMESG.RxData[0]; }
 else
	{
	 uint8_t DataPos = 1;
	 this->ModuleLongOpData.State = MMESG.RxData[DataPos + IDILONGOP_STATE_Pos];
	 this->ModuleLongOpData.Timeout = ((uint32_t)MMESG.RxData[DataPos + IDILONGOP_REMAIN_TIME_Pos + 0] << 24) |
									  ((uint32_t)MMESG.RxData[DataPos + IDILONGOP_REMAIN_TIME_Pos + 1] << 16) |
									  ((uint32_t)MMESG.RxData[DataPos + IDILONGOP_REMAIN_TIME_Pos + 2] << 8 ) |
									  ((uint32_t)MMESG.RxData[DataPos + IDILONGOP_REMAIN_TIME_Pos + 3]      );
	 return IDIER_NOPE;
	}
}
//=============================================================================================================================================================================================================
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
//=============================================================================================================================================================================================================
uint8_t IdiBusSlave::c_Dummy(uint8_t Device, uint8_t Channel)
{
 IdiBusDevice  *DEV;
 IdiBusChannel *CHNL;
 if ( (DEV = this->getDevice(Device)) == NULL )   { return IDIERMST_INVALID_DEV_NUM; }
 if ( (CHNL = DEV->getChannel(Channel)) == NULL ) { return IDIERMST_INVALID_CHN_NUM; }

 IdiBusMMES MMES(this->Address);
 MMES.ComFunc       = IDIMMES_COM_C_Dummy;
 MMES.DEV.NUM       = Device;
 MMES.CHNL.NUM      = Channel;
 
 uint8_t RequestError = this->SendRequestMMES(&MMES);
 if ( RequestError != IDIER_NOPE )  { CHNL->MMES_Error = RequestError;   }
 else if ( MMES.RxDataLength != 1 ) { CHNL->MMES_Error = IDIERMST_INVALID_RX_REQUEST_FORMAT; }
 else                               { CHNL->MMES_Error = MMES.RxData[0]; }
 return CHNL->MMES_Error;
}
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
uint8_t IdiBusSlave::c_AssignGroup(uint8_t Device, uint8_t Channel, uint8_t Group)
{
 IdiBusDevice  *DEV;
 IdiBusChannel *CHNL;
 if ( (DEV = this->getDevice(Device)) == NULL )   { return IDIERMST_INVALID_DEV_NUM; }
 if ( (CHNL = DEV->getChannel(Channel)) == NULL ) { return IDIERMST_INVALID_CHN_NUM; }
 if ( Group > IDIBUS_GROUP_LAST_NUMBER ) { CHNL->MMES_Error = IDIERMST_INVALID_TX_PARAM; }
 else
	{
 	 IdiBusMMES MMES(this->Address, &Group, 1);
	 MMES.ComFunc       = IDIMMES_COM_C_AssignGroup;
	 MMES.DEV.NUM       = Device;
	 MMES.CHNL.NUM      = Channel;
 
	 uint8_t RequestError = this->SendRequestMMES(&MMES);
	 if ( RequestError != IDIER_NOPE )  { CHNL->MMES_Error = RequestError;   }
	 else if ( MMES.RxDataLength != 1 ) { CHNL->MMES_Error = IDIERMST_INVALID_RX_REQUEST_FORMAT; }
	 else                               { CHNL->MMES_Error = MMES.RxData[0]; }
	}
 return CHNL->MMES_Error;
}
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
uint8_t IdiBusSlave::c_CheckChannelLongOp(uint8_t Device, uint8_t Channel)
{
 IdiBusDevice  *DEV;
 IdiBusChannel *CHNL;
 if ( (DEV = this->getDevice(Device)) == NULL )   { return IDIERMST_INVALID_DEV_NUM; }
 if ( (CHNL = DEV->getChannel(Channel)) == NULL ) { return IDIERMST_INVALID_CHN_NUM; }
 
 IdiBusMMES MMES(this->Address);
 MMES.ComFunc       = IDIMMES_COM_C_CheckChannelLongOp;
 MMES.DEV.NUM       = Device;
 MMES.CHNL.NUM      = Channel;
 
 uint8_t RequestError = this->SendRequestMMES(&MMES);
 if      ( RequestError != IDIER_NOPE )                           { CHNL->MMES_Error = RequestError; }
 else if ( MMES.RxDataLength != (IDILONGOP_MES_DATA_LENGTH + 1) ) { CHNL->MMES_Error = IDIERMST_INVALID_RX_REQUEST_FORMAT; }
 else if ( MMES.RxData[0] != IDIER_NOPE )                         { CHNL->MMES_Error = MMES.RxData[0]; }
 else
	{ 
	 CHNL->MMES_Error = IDIER_NOPE;
	 uint8_t DataPos = 1;
	 CHNL->LONG_OP.State = MMES.RxData[DataPos + IDILONGOP_STATE_Pos];
	 CHNL->LONG_OP.Timeout = ((uint32_t)MMES.RxData[DataPos + IDILONGOP_REMAIN_TIME_Pos + 0] << 24) |
							 ((uint32_t)MMES.RxData[DataPos + IDILONGOP_REMAIN_TIME_Pos + 1] << 16) |
							 ((uint32_t)MMES.RxData[DataPos + IDILONGOP_REMAIN_TIME_Pos + 2] << 8 ) |
							 ((uint32_t)MMES.RxData[DataPos + IDILONGOP_REMAIN_TIME_Pos + 3]      );
	}
 return CHNL->MMES_Error;
}
//=============================================================================================================================================================================================================
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
//=============================================================================================================================================================================================================
uint8_t IdiBusSlave::getAddress(void) { return this->Address; }
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
void IdiBusSlave::setAes256ToSn(uint8_t *AES256_key) { memcpy(this->ModuleST.SN.VAR.AES, AES256_key, IDISN_VARP_AES256_Length); }
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
IdiBusLongOpData IdiBusSlave::getModuleLongOpData(void) { return this->ModuleLongOpData; }
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
IdiBusLongOpData IdiBusSlave::getChLongOpData(uint8_t Device, uint8_t Channel)
{
 IdiBusDevice  *DEV;
 IdiBusChannel *CHNL;
 if ( ((DEV = this->getDevice(Device)) != NULL) && ((CHNL = DEV->getChannel(Channel)) != NULL) )
	{
	 return CHNL->LONG_OP;
	}
 IdiBusLongOpData NewData;
 return NewData; 
}
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
IdiMasterState IdiBusSlave::getModuleSN(void) { return this->ModuleST; }
//=============================================================================================================================================================================================================
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
//=============================================================================================================================================================================================================
void IdiMasterState::parseSmesDataNoAes(uint8_t *Data)
{
 this->STATE.B0S.StError      = (Data[IDISTATUS_B0S_Pos] >> IDISTATUS_B0S_ST_ERROR_Pos) & 0x01;
 this->STATE.B0S.StState      = (Data[IDISTATUS_B0S_Pos] >> IDISTATUS_B0S_ST_STATE_Pos) & IDISTATUS_B0S_ST_STATE_Msk;
 this->STATE.B0S.AesSupported = (Data[IDISTATUS_B0S_Pos] >> IDISTATUS_B0S_AES_SUPPORTED_Pos) & 0x01;
 this->STATE.B0S.AesInstalled = (Data[IDISTATUS_B0S_Pos] >> IDISTATUS_B0S_AES_INSTALLED_Pos) & 0x01;
 this->STATE.B0S.SendAlarmL0  = (Data[IDISTATUS_B0S_Pos] >> IDISTATUS_B0S_SEND_ALARM_L0_Pos) & 0x01;
 this->STATE.B0S.SendAlarmL1  = (Data[IDISTATUS_B0S_Pos] >> IDISTATUS_B0S_SEND_ALARM_L1_Pos) & 0x01;
 
 this->STATE.B1S.ModuleType      = (Data[IDISTATUS_B1S_Pos] >> IDISTATUS_B1S_MODULE_TYPE_Pos) & IDISTATUS_B1S_MODULE_TYPE_Msk;
 this->STATE.B1S.BridgeConnected = (Data[IDISTATUS_B1S_Pos] >> IDISTATUS_B1S_BRIDGE_CONNECTED_Pos) & 0x01;
 this->STATE.B1S.SelfInit        = (Data[IDISTATUS_B1S_Pos] >> IDISTATUS_B1S_SELF_INIT_Pos) & 0x01;
 this->STATE.B1S.TimeoutLed      = (Data[IDISTATUS_B1S_Pos] >> IDISTATUS_B1S_TIMEOUT_LED_Pos) & 0x01;
 this->STATE.B1S.NoMMESTimeout   = (Data[IDISTATUS_B1S_Pos] >> IDISTATUS_B1S_NO_MMES_TIMEOUT_Pos) & 0x01;
 this->STATE.B1S.CatchAlarmL0    = (Data[IDISTATUS_B1S_Pos] >> IDISTATUS_B1S_CATCH_ALARM_L0_Pos) & 0x01;
 this->STATE.B1S.CatchAlarmL1    = (Data[IDISTATUS_B1S_Pos] >> IDISTATUS_B1S_CATCH_ALARM_L1_Pos) & 0x01;
 
 memcpy(this->SN.FIX.GS1_country, &Data[IDISTATUS_SN_Pos + IDISN_FIXP_GS1_COUNTRY_Pos], IDISN_FIXP_GS1_COUNTRY_Length);
 memcpy(this->SN.FIX.GS1_company, &Data[IDISTATUS_SN_Pos + IDISN_FIXP_GS1_COMPANY_Pos], IDISN_FIXP_GS1_COMPANY_Length);
 memcpy(this->SN.FIX.ModuleType,  &Data[IDISTATUS_SN_Pos + IDISN_FIXP_MODULE_TYPE_Pos], IDISN_FIXP_MODULE_TYPE_Length);
 memcpy(this->SN.FIX.HW_rev,      &Data[IDISTATUS_SN_Pos + IDISN_FIXP_HW_REV_Pos],      IDISN_FIXP_HW_REV_Length);
 memcpy(this->SN.FIX.Serial,      &Data[IDISTATUS_SN_Pos + IDISN_FIXP_SERIAL_Pos],      IDISN_FIXP_SERIAL_Length);
 memcpy(this->SN.FIX.MAC,         &Data[IDISTATUS_SN_Pos + IDISN_FIXP_MAC_Pos],         IDISN_FIXP_MAC_Length);
 memcpy(this->SN.VAR.SW_rev,      &Data[IDISTATUS_SN_Pos + IDISN_VARP_SW_REV_Pos],      IDISN_VARP_SW_REV_Length);
 memcpy(this->SN.VAR.IPv4,        &Data[IDISTATUS_SN_Pos + IDISN_VARP_IPv4_Pos],        IDISN_VARP_IPv4_Length);
 memcpy(this->SN.VAR.IPv6,        &Data[IDISTATUS_SN_Pos + IDISN_VARP_IPv6_Pos],        IDISN_VARP_IPv6_Length);
 
 uint8_t *VerifDateP    = &Data[IDISTATUS_SN_Pos + IDISN_VARP_VERIF_DATE_Pos];
 uint8_t *VerifExpDateP = &Data[IDISTATUS_SN_Pos + IDISN_VARP_EXPIR_DATE_Pos];
 IdiDateFormat verifDate(VerifDateP[0], VerifDateP[1], VerifDateP[2], VerifDateP[3]);
 IdiDateFormat verifExpDate(VerifExpDateP[0], VerifExpDateP[1], VerifExpDateP[2], VerifExpDateP[3]);
 this->SN.VAR.VerifDate    = verifDate;
 this->SN.VAR.VerifExpDate = verifExpDate;
}
//-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
//#############################################################################################################################################################################################################
